""" This module contains main user-facing Smashrun interface.

"""
from __future__ import division

import json
import datetime
from itertools import islice

from requests_oauthlib import OAuth2Session

auth_url = "https://secure.smashrun.com/oauth2/authenticate"
token_url = "https://secure.smashrun.com/oauth2/token"


class Smashrun(object):
    def __init__(self, client_id=None, client_secret=None, client=None,
                 auto_refresh_url=None, auto_refresh_kwargs=None, scope=None,
                 redirect_uri=None, token=None, state=None, token_updater=None,
                 **kwargs):
        self.session = OAuth2Session(
            client_id=client_id,
            client=client,
            auto_refresh_url=token_url,
            scope=scope,
            redirect_uri=redirect_uri,
            token=token,
            state=state,
            token_updater=token_updater,
            **kwargs
        )
        self.client_secret = client_secret
        self.base_url = "https://api.smashrun.com/v1"

    @property
    def client_id(self):
        return self.session.client_id

    def get_auth_url(self):
        return self.session.authorization_url(
            auth_url, client_secret=self.client_secret)

    def fetch_token(self, **kwargs):
        """Fetch a new token using the supplied code.

        :param str code: A previously obtained auth code.

        """
        if 'client_secret' not in kwargs:
            kwargs.update(client_secret=self.client_secret)
        return self.session.fetch_token(token_url, **kwargs)

    def refresh_token(self, **kwargs):
        """Refresh the authentication token.

        :param str refresh_token: The refresh token to use. May be empty if
                                  retrieved with ``fetch_token``.

        """
        if 'client_secret' not in kwargs:
            kwargs.update(client_secret=self.client_secret)
        if 'client_id' not in kwargs:
            kwargs.update(client_id=self.client_id)
        return self.session.refresh_token(token_url, **kwargs)

    def get_activity(self, id_num):
        """Return the activity with the given id.

        Note that this contains more detailed information than returned
        by `get_activities`.

        """
        url = self._build_url('my', 'activities', id_num)
        return self._json(url)

    def get_activities(self, count=10, since=None, style='summary',
                       limit=None):
        """Iterate over all activities, from newest to oldest.

        :param count: The number of results to retrieve per page.
        :param since: Return only activities since this date. Can be either
                      a timestamp or a datetime object.

        :param style: The type of records to return. May be one of
                      'summary', 'briefs', 'ids', or 'extended'.

        :param limit: The maximum number of activities to return for the given
                      query.

        """
        params = {}
        if since:
            params.update(fromDate=to_timestamp(since))
        parts = ['my', 'activities', 'search']
        if style != 'summary':
            parts.append(style)
        url = self._build_url(*parts)
        # TODO: return an Activity (or ActivitySummary?) class that can do
        # things like convert date and time fields to proper datetime objects
        return islice(self._iter(url, count, **params), limit)

    def get_badges(self):
        """Return all badges the user has earned."""
        url = self._build_url('my', 'badges')
        return self._json(url)

    def get_notables(self, id_num):
        """Return the notables of the activity with the given id.
        """
        url = self._build_url('my', 'activities', id_num, 'notables')
        return self._json(url)

    def get_polyline(self, id_num, style='google'):
        """Return the polyline of the activity with the given id.

        :param style: The type of polyline to return. May be one of
                      'google', 'svg', or 'geojson'.

        """
        parts = ['my', 'activities', id_num, 'polyline']
        if style != 'google':
            parts.append(style)
        url = self._build_url(*parts)

        return self._json(url)

    def get_splits(self, id_num, unit='mi'):
        """Return the splits of the activity with the given id.

        :param unit: The unit to use for splits. May be one of
                      'mi' or 'km'.

        """
        url = self._build_url('my', 'activities', id_num, 'splits', unit)

        return self._json(url)

    def get_stats(self, year=None, month=None):
        """Return stats for the given year and month."""
        parts = ['my', 'stats']
        if month and not year:
            raise ValueError("month cannot be specified without year")
        if year:
            parts.append(year)
        if month:
            parts.append(year)
        url = self._build_url(*parts)
        return self._json(url)

    def get_current_weight(self):
        """Return the most recent weight recording."""
        url = self._build_url('my', 'body', 'weight', 'latest')
        return self._json(url)

    def get_weight_history(self):
        """Return all weight recordings."""
        url = self._build_url('my', 'body', 'weight')
        return self._json(url)

    def get_userinfo(self):
        """Return information about the current user."""
        url = self._build_url('my', 'userinfo')
        return self._json(url)

    def create_weight(self, weight, date=None):
        """Submit a new weight record.

        :param weight: The weight, in kilograms.
        :param date: The date the weight was recorded. If not
                     specified, the current date will be used.

        """
        url = self._build_url('my', 'body', 'weight')
        data = {'weightInKilograms': weight}
        if date:
            if not date.is_aware():
                raise ValueError("provided date is not timezone aware")
            data.update(date=date.isoformat())
        headers = {'Content-Type': 'application/json; charset=utf8'}
        r = self.session.post(url, data=json.dumps(data), headers=headers)
        r.raise_for_status()
        return r

    def create_activity(self, data):
        """Create a new activity (run).

        :param data: The data representing the activity you want to upload.
                     May be either JSON, GPX, or TCX.

        """
        url = self._build_url('my', 'activities')
        if isinstance(data, dict):
            data = json.dumps(data)
        r = self.session.post(url, data=data)
        r.raise_for_status()
        return r

    def update_activity(self, id_num, data):
        """Update an existing activity (run).

        :param id_num: The activity ID to update
        :param data: The data representing the activity you want to upload.
                     May be either JSON, GPX, or TCX.

        """
        url = self._build_url('my', 'activities')
        if isinstance(data, dict):
            data = json.dumps(data)
        r = self.session.put(url, data=data)
        r.raise_for_status()
        return r

    def delete_activity(self, id_num):
        """Delete an activity (run).

        :param id_num: The activity ID to delete


        """
        url = self._build_url('my', 'activities', id_num)
        r = self.session.delete(url)
        r.raise_for_status()
        return r

    def _iter(self, url, count, cls=None, **kwargs):
        page = None if count is None else 0
        while True:
            kwargs.update(count=count, page=page)
            r = self.session.get(url, params=kwargs)
            r.raise_for_status()
            data = r.json()
            if not data:
                break
            for d in data:
                if cls:
                    yield cls(d)
                else:
                    yield d
            if page is None:
                return
            page += 1

    def _build_url(self, *args, **kwargs):
        parts = [kwargs.get('base_url') or self.base_url]
        parts.extend(args)
        parts = [str(p) for p in parts]
        return "/".join(parts)

    def _json(self, url):
        response = self.session.get(url)
        response.raise_for_status()
        return response.json()


def to_timestamp(dt):
    """Convert a datetime object to a unix timestamp.

    Note that unlike a typical unix timestamp, this is seconds since 1970
    *local time*, not UTC.

    If the passed in object is already a timestamp, then that value is
    simply returned unmodified.
    """
    if isinstance(dt, int):
        return dt
    return int(total_seconds(dt.replace(tzinfo=None) -
               datetime.datetime(1970, 1, 1)))


def total_seconds(delta):
    if hasattr(delta, 'total_seconds'):
        return delta.total_seconds()
    return (delta.microseconds +
            (delta.seconds + delta.days * 24 * 3600) * 10**6) / 10**6


def is_aware(d):
    return d.tzinfo is not None and d.tzinfo.utcoffset(d) is not None



import os
import logging
import datetime

import requests
from unittest2 import TestCase, SkipTest

from smashrun import Smashrun

client_id = os.environ.get("SMASHRUN_CLIENT_ID")
client_secret = os.environ.get("SMASHRUN_CLIENT_SECRET")
refresh_token = os.environ.get("SMASHRUN_REFRESH_TOKEN")

if os.environ.get("DEBUG"):
    logging.basicConfig(level=logging.DEBUG)


class TestSmashrun(TestCase):

    """Tests for the smashrun client.

    Note that these tests are *not* unit tests and actually run against the
    live Smashrun API, so be nice and don't repeatedly run them a ridiculous
    amount.

    """

    @classmethod
    def setUpClass(cls):
        if not all((client_id, client_secret, refresh_token)):
            raise SkipTest("Missing SMASHRUN_REFRESH_TOKEN, "
                           "SMASHRUN_CLIENT_ID, and SMASHRUN_CLIENT_SECRET "
                           "environment variables")

    def setUp(self):
        self.client = Smashrun(client_id=client_id,
                               client_secret=client_secret,
                               redirect_uri='http://localhost')
        self.client.refresh_token(refresh_token=refresh_token)

    def test_get_activities(self):
        activities = list(self.client.get_activities())
        self.assertNotEqual(activities, [])  # small chance this will fail...

    def test_get_activities_style(self):
        for style in ('summary', 'briefs', 'ids'):
            activities = self.client.get_activities(count=2, style='ids')
            activity = next(activities)
            self.assertIsInstance(activity, int)

    def test_get_activities_invalid_style(self):
        with self.assertRaises(requests.HTTPError):
            list(self.client.get_activities(style='foobar'))

    def test_get_activities_limit(self):
        activities = list(self.client.get_activities(style='ids', limit=3))
        self.assertEqual(len(activities), 3)

    def test_get_activities_since(self):
        one_month_ago = (datetime.datetime.now() -
                         datetime.timedelta(days=30))
        # TODO: it would be easier to have a test for this if each date field
        # was converted to a datetime...
        list(self.client.get_activities(since=one_month_ago))

    def test_get_activity(self):
        activities = self.client.get_activities()
        activity = next(activities)
        id_num = activity['activityId']
        fetched_activity = self.client.get_activity(id_num)
        self.assertEqual(activity['activityId'],
                         fetched_activity['activityId'])
        self.assertIn('recordingValues', fetched_activity)

    def test_get_badges(self):
        # if it doesn't raise an exception, that's good enough for me
        self.client.get_badges()

    def test_get_current_weight(self):
        self.client.get_current_weight()

    def test_get_weight_history(self):
        self.client.get_weight_history()

    def test_get_stats(self):
        self.client.get_stats()

    def test_get_stats_year(self):
        self.client.get_stats(year=2014)

    def test_get_stats_year_month(self):
        self.client.get_stats(year=2015, month=2)

    def test_get_stats_month_and_no_year(self):
        with self.assertRaises(ValueError):
            self.client.get_stats(month=1)

    def test_get_auth_url(self):
        url = self.client.get_auth_url()[0]
        self.assertIn('client_id', url)
        self.assertIn('client_secret', url)
        self.assertIn('redirect_uri', url)

    def test_get_userinfo(self):
        r = self.client.get_userinfo()
        self.assertIn('id', r)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup

# workaround for python issue #15881
try:
    import multiprocessing
except ImportError:
    pass


with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read().replace('.. :changelog:', '')

requirements = [
    'requests',
    'requests-oauthlib'
]

test_requirements = ['unittest2']

setup(
    name='smashrun-client',
    version='0.6.0',
    description="A Python client for the Smashrun API",
    long_description=readme + '\n\n' + history,
    author="Ryan Campbell",
    author_email='campbellr@gmail.com',
    url='https://github.com/campbellr/smashrun-client',
    packages=[
        'smashrun',
    ],
    package_dir={'smashrun':
                 'smashrun'},
    include_package_data=True,
    install_requires=requirements,
    license="Apache",
    zip_safe=False,
    keywords='smashrun',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: Apache Software License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    test_suite='unittest2.collector',
    tests_require=test_requirements
)
